//************************************************************************
//  Buttefly Class.
//     This class animates a buttefly.
//************************************************************************
//


import com.jogamp.opengl.util.*;
import javax.media.opengl.*;
import javax.media.opengl.glu.*;
import java.util.*;
import java.io.IOException;


public class Butterfly 
{
  private int butterfly_obj, left_wing, right_wing;
  private int texture;
  private float x,y,z; // position
  private GLU glu = new GLU();
  
  private float dir_x, dir_y, dir_z;
  private float movement_speed;
  
  private float angle;
  private float rot_dir;
  private float rot_speed;

  public Butterfly(float _x, float _y, float _z )
  {
    x=_x;
    y=_y;
    z=_z;
    butterfly_obj = left_wing = right_wing = 0;
    
    dir_x=1;
    dir_y=0;
    dir_z=0;
    
    movement_speed=0.05f;
    
    angle = 0;
    rot_speed = 2;
    rot_dir=1;
    
  }

  public void init( GL2 gl )
  {
    try {
        texture = load_texture( gl, "Butterfly1.bmp" );
    } catch (IOException e) {
        System.out.println("Unable to load textures,Bailing!");
        throw new RuntimeException(e);
    } 

    create_left_wing( gl, texture );
    create_right_wing( gl, texture );

    butterfly_obj = gl.glGenLists(1);

    gl.glNewList( butterfly_obj, GL2.GL_COMPILE );
    construct_disp_list( gl ); 
    gl.glEndList();
  }

  public void update( GL2 gl )
  {
	translate();
	flap_wings();
	
    gl.glNewList( butterfly_obj, GL2.GL_COMPILE );
    construct_disp_list( gl ); 
    gl.glEndList();
  }

  public void draw( GL2 gl )
  {
    gl.glPushMatrix();
    gl.glCallList( butterfly_obj );
    gl.glPopMatrix();
  }

  private void construct_disp_list( GL2 gl)
  { 
	gl.glTranslatef(x, y, z);
    gl.glBindTexture(GL.GL_TEXTURE_2D, texture);
    
    gl.glPushMatrix();
    	gl.glRotatef(angle, 1, 1, 0);
    	gl.glCallList( left_wing );
    gl.glPopMatrix();
    
    gl.glPushMatrix();
    	gl.glRotatef(-angle, 1, 1, 0);
    	gl.glCallList( right_wing );
    gl.glPopMatrix();
  }

  private void flap_wings()
  {
	   if (angle > 50) rot_dir=-1;
	   if (angle < -50) rot_dir=1;
	   
	   angle+=rot_dir*rot_speed;
  }

  private void translate()
  {
	  if (x>2) dir_x=-1;
	  if (x<-2) dir_x=1;
	  
	  x += dir_x*movement_speed;
  }

  private int load_texture( GL gl, String filename) 
    throws IOException 
  {
    TextureReader.Texture texture =TextureReader.readTexture( filename );
    int[] texture_id = new int[1];
    gl.glGenTextures(1, texture_id, 0);
 
    //Create Linear Filtered Texture
    gl.glBindTexture(GL.GL_TEXTURE_2D, texture_id[0]);
    //gl.glTexParameteri(GL.GL_TEXTURE_2D, 
    //    GL.GL_TEXTURE_MAG_FILTER, GL.GL_LINEAR);
	gl.glTexParameteri(GL.GL_TEXTURE_2D, 
	GL.GL_TEXTURE_MIN_FILTER, GL.GL_LINEAR);

	gl.glTexImage2D(GL.GL_TEXTURE_2D,
	   0,
	   3,
	   texture.getWidth(),
	   texture.getHeight(),
	   0,
	   GL.GL_RGB,
	   GL.GL_UNSIGNED_BYTE,
	   texture.getPixels());

       return texture_id[0];
  }

  private void create_left_wing( GL2 gl, int texture)
  {
    left_wing = gl.glGenLists(1);
    
    gl.glNewList( left_wing, GL2.GL_COMPILE);
    gl.glPushMatrix();
    gl.glEnable( GL.GL_TEXTURE_2D);
    gl.glPolygonMode( GL.GL_FRONT, GL2.GL_FILL);
    gl.glPolygonMode( GL.GL_BACK, GL2.GL_FILL);
    gl.glScalef(0.5f,0.5f,0.5f);
    gl.glTranslatef(-0.5f, -0.5f, 0);
    gl.glBegin( GL.GL_TRIANGLES );	  
      gl.glTexCoord2f(1.0f,1.0f); 
      gl.glVertex3f( 1.0f, 1.0f, 0.0f);
      gl.glTexCoord2f(0.0f,1.0f); 
      gl.glVertex3f( 0.0f, 1.0f, 0.0f);
      gl.glTexCoord2f(0.0f,0.0f); 
      gl.glVertex3f( 0.0f, 0.0f, 0.0f);
    gl.glEnd();
    gl.glDisable( GL.GL_TEXTURE_2D );
    gl.glPopMatrix();
    gl.glEndList();
  }

  private void create_right_wing( GL2 gl, int texture )
  {
    right_wing = gl.glGenLists(1);

    gl.glNewList( right_wing, GL2.GL_COMPILE);
    gl.glPushMatrix();
    gl.glEnable( GL.GL_TEXTURE_2D );
    gl.glPolygonMode( GL.GL_FRONT, GL2.GL_FILL);
    gl.glPolygonMode( GL.GL_BACK, GL2.GL_FILL);
    gl.glScalef(0.5f,0.5f,0.5f);
    gl.glTranslatef(-0.5f, -0.5f, 0);
    gl.glBegin( GL.GL_TRIANGLES );
      gl.glTexCoord2f(1.0f,1.0f); 
      gl.glVertex3f( 1.0f, 1.0f, 0.0f);
      gl.glTexCoord2f(0.0f,0.0f); 
      gl.glVertex3f( 0.0f, 0.0f, 0.0f);
      gl.glTexCoord2f(1.0f,0.0f); 
      gl.glVertex3f( 1.0f, 0.0f, 0.0f);
    gl.glEnd();
    gl.glDisable( GL.GL_TEXTURE_2D );
    gl.glPopMatrix();
    gl.glEndList();
  }

}
